@@ -84,7 +84,8 @@ public:
   bool isTypeEvent() const;
   bool isTypeDeviceEvent() const;
   bool isTypeReserveId() const;
-  bool isTypeFloat(unsigned Bits = 0) const;
+  bool isTypeFloat(unsigned Bits = 0,
+                   unsigned FloatingPointEncoding = FPEncodingMax) const;
   bool isTypeImage() const;
   bool isTypeOCLImage() const;
   bool isTypePipe() const;
@@ -93,6 +94,7 @@ public:
   bool isTypeOpaque() const;
   bool isTypePointer() const;
   bool isTypeSampler() const;
+  bool isTypeSampledImage() const;
   bool isTypeStruct() const;
   bool isTypeVector() const;
   bool isTypeJointMatrixINTEL() const;
@@ -200,16 +202,31 @@ class SPIRVTypeFloat : public SPIRVType {
 public:
   static const Op OC = OpTypeFloat;
   // Complete constructor
-  SPIRVTypeFloat(SPIRVModule *M, SPIRVId TheId, unsigned TheBitWidth)
-      : SPIRVType(M, 3, OC, TheId), BitWidth(TheBitWidth) {}
+  SPIRVTypeFloat(SPIRVModule *M, SPIRVId TheId, unsigned TheBitWidth,
+                 unsigned TheFloatingPointEncoding)
+      : SPIRVType(M, 3 + (TheFloatingPointEncoding != FPEncodingMax), OC,
+                  TheId),
+        BitWidth(TheBitWidth), FloatingPointEncoding(TheFloatingPointEncoding) {
+  }
   // Incomplete constructor
-  SPIRVTypeFloat() : SPIRVType(OC), BitWidth(0) {}
+  SPIRVTypeFloat()
+      : SPIRVType(OC), BitWidth(0), FloatingPointEncoding(FPEncodingMax) {}
 
   unsigned getBitWidth() const { return BitWidth; }
 
+  unsigned getFloatingPointEncoding() const { return FloatingPointEncoding; }
+
+  std::optional<ExtensionID> getRequiredExtension() const override {
+    if (isTypeFloat(16, FPEncodingBFloat16KHR))
+      return ExtensionID::SPV_KHR_bfloat16;
+    return {};
+  }
+
   SPIRVCapVec getRequiredCapability() const override {
     SPIRVCapVec CV;
-    if (isTypeFloat(16)) {
+    if (isTypeFloat(16, FPEncodingBFloat16KHR)) {
+      CV.push_back(CapabilityBFloat16TypeKHR);
+    } else if (isTypeFloat(16)) {
       CV.push_back(CapabilityFloat16Buffer);
       auto Extensions = getModule()->getSourceExtension();
       if (std::any_of(Extensions.begin(), Extensions.end(),
@@ -221,14 +238,34 @@ public:
   }
 
 protected:
-  _SPIRV_DEF_ENCDEC2(Id, BitWidth)
+  void encode(spv_ostream &O) const override {
+    assert(WordCount == 3 || WordCount == 4);
+    auto Encoder = getEncoder(O);
+    Encoder << Id << BitWidth;
+    if (WordCount == 4)
+      Encoder << FloatingPointEncoding;
+  }
+
+  void decode(std::istream &I) override {
+    assert(WordCount == 3 || WordCount == 4);
+    auto Decoder = getDecoder(I);
+    Decoder >> Id >> BitWidth;
+    if (WordCount == 4)
+      Decoder >> FloatingPointEncoding;
+  }
+
   void validate() const override {
     SPIRVEntry::validate();
     assert(BitWidth >= 16 && BitWidth <= 64 && "Invalid bit width");
+    assert(
+        (FloatingPointEncoding == FPEncodingMax ||
+         (BitWidth == 16 && FloatingPointEncoding == FPEncodingBFloat16KHR)) &&
+        "Invalid floating point encoding");
   }
 
 private:
   unsigned BitWidth; // Bit width
+  unsigned FloatingPointEncoding;
 };
 
 class SPIRVTypePointer : public SPIRVType {
@@ -1065,9 +1102,13 @@ class SPIRVTypeJointMatrixINTEL : public SPIRVType {
   std::vector<SPIRVValue *> Args;
 
 public:
-  const static Op OC = internal::OpTypeJointMatrixINTEL;
   const static SPIRVWord FixedWC = 3;
-  // Complete constructor
+  // Complete constructor with non-default OC
+  SPIRVTypeJointMatrixINTEL(SPIRVModule *M, SPIRVId TheId, Op OC,
+                            SPIRVType *CompType,
+                            std::vector<SPIRVValue *> Args);
+
+  // Incomplete constructor for default OC
   SPIRVTypeJointMatrixINTEL(SPIRVModule *M, SPIRVId TheId, SPIRVType *CompType,
                             std::vector<SPIRVValue *> Args);
   // Incomplete constructor
@@ -1086,9 +1127,30 @@ public:
   SPIRVType *getCompType() const { return CompType; }
   SPIRVValue *getRows() const { return Args[0]; }
   SPIRVValue *getColumns() const { return Args[1]; }
-  SPIRVValue *getLayout() const { return Args[2]; }
-  SPIRVValue *getScope() const { return Args[3]; }
-  SPIRVValue *getUse() const { return Args.size() > 4 ? Args[4] : nullptr; }
+
+  SPIRVValue *getLayout() const {
+    if (this->getOpCode() == internal::OpTypeJointMatrixINTEL)
+      return Args[2];
+    return nullptr;
+  }
+
+  SPIRVValue *getScope() const {
+    if (this->getOpCode() == internal::OpTypeJointMatrixINTEL)
+      return Args[3];
+    return Args[2];
+  }
+
+  SPIRVValue *getUse() const {
+    if (this->getOpCode() == internal::OpTypeJointMatrixINTEL)
+      return Args.size() > 4 ? Args[4] : nullptr;
+    return Args[3];
+  }
+
+  SPIRVValue *getComponentTypeInterpretation() const {
+    if (this->getOpCode() == internal::OpTypeJointMatrixINTEL)
+      return Args.size() > 5 ? Args[5] : nullptr;
+    return Args.size() > 4 ? Args[4] : nullptr;
+  }
 };
 
 class SPIRVTypeCooperativeMatrixKHR : public SPIRVType {
@@ -1109,7 +1171,10 @@ public:
     return ExtensionID::SPV_KHR_cooperative_matrix;
   }
   SPIRVCapVec getRequiredCapability() const override {
-    return getVec(CapabilityCooperativeMatrixKHR);
+    auto CV = getVec(CapabilityCooperativeMatrixKHR);
+    if (CompType->isTypeFloat(16, FPEncodingBFloat16KHR))
+      CV.push_back(CapabilityBFloat16CooperativeMatrixKHR);
+    return CV;
   }
 
   SPIRVType *getCompType() const { return CompType; }
